<?php

/**
 * @author Jan Hruby
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package moodle multiauth
 *
 * Authentication Plugin: WSTrust Authentication
 *
 * Authentication using WSTrust SSO.
 *
 * 2012-10-24  File created.
 */

if (!defined('MOODLE_INTERNAL')) {
    die('Direct access to this script is forbidden.');    ///  It must be included from a Moodle page
}

require_once($CFG->libdir.'/authlib.php');


class WSTCredentials
{
  function WSTCredentials($username)
  {
    $this->username = $username; 
    $this->password = "";
  }
}
/**
 * WSTrust authentication plugin.
 */
class auth_plugin_wstrust extends auth_plugin_base {
     function init_plugin($authtype) {
        $this->pluginconfig = 'auth/'.$authtype;
        $this->config = get_config($this->pluginconfig);
     }
    /**
     * Constructor.
     */
    function auth_plugin_wstrust() {
        $this->authtype = 'wstrust';
        $this->roleauth = 'auth_wstrust';
        $this->errorlogtag = '[AUTH WSTRUST] ';
        $this->init_plugin($this->authtype);
    }

    function prevent_local_passwords() {
        return true;
    }

    /**
     * Authenticates user against WSTrust
     * Returns true if the username and password work and false if they are
     * wrong or don't exist.
     *
     * @param string $username The username (with system magic quotes)
     * @param string $password The password (with system magic quotes)
     * @return bool Authentication success or failure.
     */
    function user_login ($username, $password) {               
        return false;
    }

    /**
     * Returns true if this authentication plugin is 'internal'.
     *
     * @return bool
     */
    function is_internal() {
        return false;
    }

    /**
     * Returns true if this authentication plugin can change the user's
     * password.
     *
     * @return bool
     */
    function can_change_password() {
        return false;
    }

    /**
     * Authentication choice (WSTrust or other)
     * Redirection to the WSTrust form or to login/index.php
     * for other authentication
     */
    function loginpage_hook() {
      global $CFG;
      
      if ($_SERVER['REQUEST_METHOD'] === 'GET')
      {
        $ip = $this->WSTGetClientIP();
        $username =  $this->WSTobtainID($this->config->wstrustURL, $ip, "screenName");
        
        if (!empty($username))
        {
          $GLOBALS["frm"] = new WSTCredentials($username);
          $user =  get_complete_user_data('username', $username, $CFG->mnet_localhost_id);
          
                     

          if (!$user)
          {
             if (empty($CFG->authpreventaccountcreation)) {
                $user = create_user_record($username, "", "ldap");
            } 
          }
          else
          {
            if (!empty($user->suspended)) {
              add_to_log(SITEID, 'login', 'error', 'index.php', $username);
              error_log('[client '.getremoteaddr()."]  $CFG->wwwroot  Suspended Login:  $username  ".$_SERVER['HTTP_USER_AGENT']);
              $user = false;
            }
          }
          
          
          $GLOBALS["user"] = $user;
        }
      }
    }

    /**
     * Prints a form for configuring this authentication plugin.
     *
     * This function is called from admin/auth.php, and outputs a full page with
     * a form for configuring this plugin.
     *
     * @param array $page An object containing all the data for this page.
     */
    function config_form($config, $err, $user_fields) {
        global $CFG, $OUTPUT;

        if (!function_exists('curl_init')) { // Is curl really there?
            echo $OUTPUT->notification(get_string('curl_noextension', 'auth_wstrust'));
        }
        
         if (!function_exists('json_decode'))
         {
            echo $OUTPUT->notification(get_string('json_decode_noextension', 'auth_wstrust'));

         }

        include($CFG->dirroot.'/auth/wstrust/config.html');
    }

    
    /**
     * A chance to validate form data, and last chance to
     * do stuff before it is inserted in config_plugin
     * @param object object with submitted configuration settings (without system magic quotes)
     * @param array $err array of error messages
     */
    function validate_form($form, &$err) {
      $json_url = $form->wstrustURL.'/json/userByIP/test';
      $ch = curl_init( $json_url );
    
      $options = array(
        CURLOPT_RETURNTRANSFER => true
        );
    
      curl_setopt_array( $ch, $options );
    
      $result =  curl_exec($ch);
    
      if ($result) {
        $obj = json_decode($result,true); 
        if (!array_key_exists('screenName', $obj)) {
                $err['wstrustURL'] = get_string('err_invalid_server', 'auth_wstrust');
        }
      }
      else
      {
        $err['wstrustURL'] = get_string('err_unable_to_connect', 'auth_wstrust');
      }
    }
    /**
     * Returns the URL for changing the user's pw, or empty if the default can
     * be used.
     *
     * @return moodle_url
     */
    function change_password_url() {
        return null;
    }

    /**
     * Processes and stores configuration data for this authentication plugin.
     */
    function process_config($config) {
        // WSTrust settings
        if (!isset($config->wstrustURL)) {
            $config->wstrustURL = '';
        }
               
        // save WSTrust settings
        set_config('wstrustURL', trim($config->wstrustURL), $this->pluginconfig);       
        return true;
    }


    function iscreator($username) {
        return false;
    }


    function get_userinfo($username) {
        if (empty($this->config->host_url)) {
            return array();
        }
        return parent::get_userinfo($username);
    }


    function sync_users($do_updates=true) {
    }
  
  function WSTGetClientIP() {
    if ( isset($_SERVER["REMOTE_ADDR"]) ) {
      return $_SERVER["REMOTE_ADDR"] ;
    } else if ( isset($_SERVER["HTTP_X_FORWARDED_FOR"]) ) {
      return $_SERVER["HTTP_X_FORWARDED_FOR"] ;
    } else if ( isset($_SERVER["HTTP_CLIENT_IP"]) ) {
      return $_SERVER["HTTP_CLIENT_IP"] ;
    }
	
    return null;
  }

  function WSTobtainID($wstUrl, $ip, $id) {
    $json_url = $wstUrl.'/json/userByIP/'.$ip;
    $ch = curl_init( $json_url );
    
    $options = array(
      CURLOPT_RETURNTRANSFER => true
      );
    
    curl_setopt_array( $ch, $options );
    
    $result =  curl_exec($ch);
    
    if ($result) {
      $obj = json_decode($result,true);
      if ($obj[$id]) {
        return $obj[$id];
      }
      else {
        return null;
      }
    }
    return null;
  }
}
